/*======================PROCEDIMENTO CRIA_VENCIMENTO============================
   PROCEDIMENTO CRIA ,A PARTIR DA PARAMETRIZAO DA FORMA DE PAGAMENTO,
   VENCIMENTOS PARA O VALOR DA NOTA FISCAL QUE  PASSADO COMO PARMETRO.
   O PROCEDIMENTO DIFERENCIA FORMAS DE PAGAMENTO DE CLIENTES, FORNECEDORES,
   PEDIDOS E AUTORIZAO DE FORNECIMENTO, DE ACORDO COM A ORIGEM DA NOTA FISCAL.

   OBSERVAO :
      OS VENCIMENTOS GERADOS SERO GRAVADOS EM UMA TABELA TEMPORRIA,
      A TRAB_VENCIMENTO, SENDO QUE ESSAS INFORMAES SERO APAGADAS POSTERIORMENTE
      PELA ROTINA QUE A CHAMOU OU PELO ROLLBACK DA TRANSAO.
   -------
   ENTRADA
   -------
      O PROCEDIMENTO RECEBE OS SEGUINTES PARMETROS DE ENTRADA:
      P_CDFILIAL       : CDIGO DA FILIAL DE LANAMENTO DA NOTA FISCAL
      P_NRLANCTONF     : NMERO DE LANAMENTO DA NOTA FISCAL
      P_IDENTRSAIDOP   : 'E' - NOTA FISCAL DE ENTRADA LANADA PELO MDULO DFE
                         'S' - NOTA FISCAL DE SADA LANADA PELO MDULO DFS
                         'A' - NOTA FISCAL IMPORTADA DE UMA AF
                         'P' - NOTA FISCAL ORIGINADA DA EXPEDIO DE UM PEDIDO
      P_VRPAGTO        : VALOR DA NOTA FISCAL (CONSIDERANDO TODOS OS DESCONTOS)
      P_DTEMISSAO      : DATA DE EMISSO DA NOTA FISCAL
      P_DTENTRSAID     : DATA DE ENTRADA/SADA DA NOTA FISCAL
      P_CDCLIEFORN     : CDIGO DO CLIENTE/FORNECEDOR DA NOTA FISCAL
      P_CDFILIEMIRECE  : CDIGO DA FILIAL DE RECEBIMENTO/EMISSO DA NOTA FISCAL
      P_ORIGDESTNF     : IDENTIFICADOR DE ORIGEM DA NOTA FISCAL
      P_CDFILLANC      : (OPCIONAL)CDIGO DA FILIAL DE LANAMENTO DA AF*
      P_CDFILDEST      : (OPCIONAL)CDIGO DA FILIAL DE DESTINO DA AF*
      P_NRAUTOFORN     : (OPCIONAL)NMERO DA AF
      P_CDFILPEDIDO    : (OPCIONAL)CDIGO DA FILIAL DE LANAMENTO DO PEDIDO**
      P_NRPEDIDO       : (OPCIONAL)NMERO DO PEDIDO**

      * : SOMENTE INFORMADOS PARA NOTA FISCAL ORIUNDA DE AF
      **: SOMENTE INFORMADOS PARA NOTA FISCAL ORIUNDA DA EXPEDIO DE PEDIDOS

   -----
   SADA
   -----
      O PROCEDIMENTO NO POSSUI RETORNO
   ------------------
   MTODOS AUXILIARES
   ------------------
      FORMAPGTOFIXA : DETERMINA O VENCIMENTO A PARTIR DA FORMA FIXA DE PAGAMENTO
      DIAPGTO : DETERMINA O DIA DA SEMANA OU MS EM QUE O VENCIMENTO DEVE CAIR
      FORMAPGTO : DETERMINA A DATA BASE PARA OS VENCIMENTOS
      CONSISTE_FERI_BANC : CORRIGE O VENCIMENTO, CONSIDERANDO O FERIADO BANCRIO
      SET_PARAMETROS : INICIALIZA OS PARMETROS NECESSRIOS PARA
                       GERAO DOS VENCIMENTOS, DATA BASE, QUANTIDADE DE DIAS,
                       FORMA DE PAGAMENTO, ETC

*/
CREATE OR REPLACE PROCEDURE CRIA_VENCIMENTO(
          P_CDFILIAL      IN VARCHAR2,
          P_NRLANCTONF    IN VARCHAR2,
          P_IDENTRSAIDOP  IN VARCHAR2,
          P_VRPAGTO       IN FLOAT,
          P_DTEMISSAO     IN DATE,
          P_DTENTRSAID    IN DATE,
          P_CDCLIEFORN    IN VARCHAR2,
          P_CDFILIEMIRECE IN VARCHAR2,
          P_ORIGDESTNF    IN VARCHAR2,
          P_CDFILLANC     IN VARCHAR2,
          P_CDFILDEST     IN VARCHAR2,
          P_NRAUTOFORN    IN VARCHAR2,
          P_CDFILPEDIDO   IN VARCHAR2,
          P_NRPEDIDO      IN VARCHAR2)
AS
   V_IDFORMAPGTO     VARCHAR2(01);
   V_NRPARCPGTO      NUMBER;
   V_IDTIPODTPGT     VARCHAR2(4);

   V_SEQUENCIAL      NUMBER;
   V_VRPARCVENC      NUMBER;
   V_NRPARCELAS      NUMBER;
   V_VRPARCAUXI      NUMBER;
   V_INDIAS          NUMBER;
   V_INGUARDANRDIAS  NUMBER;

   V_DTVENCIMENTO    DATE;
   V_DTBASE          DATE;

   V_VRPAGTO         NUMBER;

   V_CDSERVCONT      VARCHAR2(5);

   V_CODIGO VARCHAR2(1);
   V_CDCLIEFORN VARCHAR2(21);

   V_IDGERPARCST  VARCHAR2(1);

   V_VRTOTIMPONF NUMBER;

   V_BOCONSFERIBANC BOOLEAN;

   V_IDTPPERIODO VARCHAR2(1);

   V_CDFORMPGTO VARCHAR2(2);
   
   CURSOR C_PARCELAS (P_CLIEFORN VARCHAR2,
                      P_CODIGO VARCHAR2,
                      P_CDSERVCONT VARCHAR2,
                      P_CDFILIDEST VARCHAR2,
                      P_CDFILILANC VARCHAR2,
                      P_NRAUTOFORN VARCHAR2,
                      P_CDFILPEDIDO VARCHAR2,
                      P_NRPEDIDO VARCHAR2)
       IS
      SELECT NRDIASPGTO, VRPERCPGTO
        FROM
      (SELECT NVL(NRDIASPGTOFO, 0) AS NRDIASPGTO, NVL(VRPERCPGTOFO, 100) VRPERCPGTO
        FROM PRAZOPGTFORN
       WHERE CDFORNECED = P_CDCLIEFORN
         AND P_CODIGO = '0'
       UNION
      SELECT NVL(NRDIASPGTOCL, 0) AS NRDIASPGTO, NVL(VRPERCPGTOCL, 100) VRPERCPGTO
        FROM PRAZOPGTCLIE
       WHERE CDCLIENTE = P_CDCLIEFORN
         AND P_CODIGO = '1'
       UNION
      SELECT NVL(NRDIASPGTOSC, 0) AS NRDIASPGTO, NVL(VRPERCPGTOSC, 100) VRPERCPGTO
        FROM PZPGSERCONT
       WHERE CDCLIENTE = P_CDCLIEFORN
         AND CDSERVCONT = P_CDSERVCONT
         AND P_CODIGO = '2'
       UNION
      SELECT NVL(NRDIASPGTOAF, 0) AS NRDIASPGTO, NVL(VRPERCPGTOAF, 100) VRPERCPGTO
        FROM PRAZOPGTAF
       WHERE CDFILDEST = P_CDFILDEST
         AND CDFILLANC  = P_CDFILLANC
         AND NRAUTOFORN = P_NRAUTOFORN
         AND P_CODIGO = '3'
       UNION
      SELECT NVL(NRDIASPGTOPED, 0) AS NRDIASPGTO, NVL(VRPERCPGTOPED, 100) VRPERCPGTO
        FROM PRAZOPGTOPED
       WHERE CDFILIAL = P_CDFILPEDIDO
         AND NRPEDIDO = P_NRPEDIDO
         AND P_CODIGO = '4');

   FUNCTION FORMAPGTOFIXA(P_DTBASE      IN DATE,
                          P_IDUTDIAPGTO IN VARCHAR2,
                          V_NRDIAFIXO   IN VARCHAR2)
            RETURN DATE
   IS
      V_DATEVENCI    DATE;
      V_INDIAS       NUMBER;

   BEGIN
      IF NOT (P_DTBASE IS NULL) THEN
         /* FORMA FIXA Verifica se o numero informado no cadastro de forma de pagamento  maior
         ou igual ao dia da data de vencimento. Ou se o opo de gerar 1 parcela no proximo mes
         esteja marcada  */
         IF (P_IDUTDIAPGTO = 'I') THEN
            IF (V_NRDIAFIXO >= TO_NUMBER(TO_CHAR(P_DTBASE, 'DD'))) THEN
               /* pega o maior dia do mes */
               V_INDIAS :=  TO_NUMBER(TO_CHAR(LAST_DAY(P_DTBASE), 'DD'));

                /* Verifica se numero informado no cadastro de forma de pagamento 
                   maior que o ultimo dia do mes. Exemplo informou 31 no cadastro e o ultimo dia do e 30*/
               IF V_NRDIAFIXO > V_INDIAS THEN
                  /*Pega o maior dia do mes */
                  V_DATEVENCI := LAST_DAY(P_DTBASE);
               ELSE
                  /*Joga a data com o dia informado na parametrizcao */
                  V_DATEVENCI := TO_DATE(TO_CHAR(V_NRDIAFIXO)||'/'||TO_CHAR(P_DTBASE, 'MM/YYYY'), 'DD/MM/YYYY');
               END IF;
            ELSE
               V_DATEVENCI := ADD_MONTHS(P_DTBASE, 1);
               V_DATEVENCI := TO_DATE(TO_CHAR(V_NRDIAFIXO)||'/'||TO_CHAR(V_DATEVENCI, 'MM/YYYY'), 'DD/MM/YYYY');
            END IF;
         ELSE
            V_DATEVENCI := ADD_MONTHS(P_DTBASE, 1);
            V_DATEVENCI := TO_DATE(TO_CHAR(V_NRDIAFIXO)||'/'||TO_CHAR(V_DATEVENCI, 'MM/YYYY'), 'DD/MM/YYYY');
         END IF;
         RETURN(V_DATEVENCI);
      ELSE
         RETURN(P_DTEMISSAO);
      END IF;
   END;
   /*DETERMINA O DIA DA SEMANA OU DO MS EM QUE O PAGAMENTO DEVE CAIR*/
   FUNCTION DIAPGTO (P_DTBASE IN DATE,
                     P_IDTPPERIODO IN VARCHAR2,
                     P_CDFORMPGTO IN VARCHAR2) RETURN DATE
   IS
      V_DTVENCIMENTO DATE;
   BEGIN
      V_DTVENCIMENTO := P_DTBASE;
      IF P_IDTPPERIODO = 'M' THEN
         V_DTVENCIMENTO := UTIL_CRIAVENCIMENTO.DIAMES(P_CDFORMPGTO,
                                                      P_DTBASE);
      ELSIF p_IDTPPERIODO = 'S' THEN
         V_DTVENCIMENTO := UTIL_CRIAVENCIMENTO.DIASEMANA(P_CDFORMPGTO,
                                                         P_DTBASE);
      ELSE
         V_DTVENCIMENTO := UTIL_CRIAVENCIMENTO.DIASEMANAMES(P_CDFORMPGTO,
                                                            P_DTBASE);
      END IF;
      RETURN V_DTVENCIMENTO;
   END;
   /*DETEMINA A DATA BASE E O NMERO DE DIAS A PARTIR DO TIPO DA FORMA DE PAGAMENTO
   E DO TIPO DA DATA DE PAGAMENTO*/
   PROCEDURE FORMAPGTO(P_INDIAS OUT NUMBER,
                       P_DTBASE OUT DATE,
                       P_DTEMISSAO IN DATE,
                       P_DTENTRSAID IN DATE,
                       P_IDTPFORPGT IN VARCHAR2,
                       P_IDTIPODTPGT IN VARCHAR2,
                       P_IDUTDIAPGT IN VARCHAR2,
                       P_NRDIAFIXO IN VARCHAR2,
                       P_CODIGO IN VARCHAR2,
                       P_IDTPPERIODO IN VARCHAR2,
                       P_CDFORMPGTO IN VARCHAR2)
   IS
   V_DTAUX DATE;
   BEGIN
      IF NOT P_IDTPFORPGT IS NULL THEN
         IF P_IDTPFORPGT = 'V' THEN
            IF P_IDTIPODTPGT = 'EMIN' THEN
               P_INDIAS := 0;
               P_DTBASE := P_DTEMISSAO;
            ELSIF P_IDTIPODTPGT = 'EMEX' THEN
               P_INDIAS := 1;
               P_DTBASE := P_DTEMISSAO;
            ELSIF P_IDTIPODTPGT = 'ENIN' THEN
               P_INDIAS := 0;
               P_DTBASE := P_DTENTRSAID;
            ELSIF P_IDTIPODTPGT = 'ENEX' THEN
               P_INDIAS := 1;
               P_DTBASE := P_DTENTRSAID;
            ELSIF P_IDTIPODTPGT = 'UEMI' THEN
               P_INDIAS := 0;
               P_DTBASE := P_DTEMISSAO;
            ELSIF P_IDTIPODTPGT = 'UEME' THEN
               P_INDIAS := 1;
               P_DTBASE := P_DTEMISSAO;
            END IF;
            V_DTAUX := P_DTBASE + P_INDIAS;
            V_DTAUX := V_DTAUX - 1;
            /*SE FOR NOTA FISCAL DE SERVIO, CONSIDERA O DIA DE PAGAMENTO*/
            IF (P_CODIGO = '4') AND NOT (P_CDFORMPGTO IS NULL) THEN
               V_DTAUX := DIAPGTO(V_DTAUX,
                                  P_IDTPPERIODO,
                                  P_CDFORMPGTO);
            END IF;
         ELSE
            P_DTBASE := P_DTEMISSAO;
            V_DTAUX := FORMAPGTOFIXA(P_DTEMISSAO ,P_IDUTDIAPGT, P_NRDIAFIXO);
         END IF;
         P_INDIAS := TRUNC(V_DTAUX - P_DTBASE);
         IF (P_INDIAS IS NULL) THEN
            P_INDIAS := 0;
         END IF;
      END IF;
   END;

   /*CORRIGE A DATA DE VENCIMENTO CONSIDERANDO FERIADO BANCRIO*/
   PROCEDURE CONSISTE_FERI_BANC(P_CDFILIEMIRECE IN VARCHAR2,
                                P_DTVENCIMENTO IN DATE,
                                P_DTVENC OUT DATE,
                                P_IDTIPODTPGTO IN VARCHAR2,
                                P_DTEMISSAO IN DATE)
   IS
      V_BOFERIFILI BOOLEAN;
      V_NRDIASFERI NUMBER;
      V_DTANTERIOR DATE;
   BEGIN
      V_BOFERIFILI := TRUE;
      P_DTVENC := P_DTVENCIMENTO;
      WHILE V_BOFERIFILI LOOP
         IF P_IDTIPODTPGTO = 'UEME' THEN
            BEGIN
               SELECT COUNT(*)
                 INTO V_NRDIASFERI
                 FROM FERIFILI
                WHERE CDFILIAL = P_CDFILIEMIRECE
                  AND DTFERIFILI BETWEEN P_DTEMISSAO+1 AND P_DTVENC;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_NRDIASFERI := 0;
            END;
         ELSE
            BEGIN
               SELECT COUNT(*)
                 INTO V_NRDIASFERI
                 FROM FERIFILI
                WHERE CDFILIAL = P_CDFILIEMIRECE
                  AND DTFERIFILI BETWEEN P_DTEMISSAO AND P_DTVENC;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_NRDIASFERI := 0;
            END;
         END IF;

         IF NOT(V_NRDIASFERI = 0) THEN
            V_DTANTERIOR := P_DTVENC;
            P_DTVENC := P_DTVENC + V_NRDIASFERI;
            WHILE V_BOFERIFILI LOOP
               V_DTANTERIOR := V_DTANTERIOR+1;
               BEGIN
                  SELECT COUNT(*)
                    INTO V_NRDIASFERI
                    FROM FERIFILI
                   WHERE CDFILIAL = P_CDFILIEMIRECE
                     AND DTFERIFILI BETWEEN V_DTANTERIOR AND P_DTVENC;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_NRDIASFERI := 0;
               END;

               IF V_NRDIASFERI > 0 THEN
                  V_DTANTERIOR := P_DTVENC;
                  P_DTVENC := P_DTVENC + V_NRDIASFERI;
               ELSE
                  V_BOFERIFILI := FALSE;
               END IF;
            END LOOP;
         ELSE
            V_BOFERIFILI := FALSE;
         END IF;
      END LOOP;
   END;

   /*ALTERAO DOS PARMETROS NECESSRIOS PARA GERAO DO FINANCEIRO
     QUANTIDADE DE DIAS, DATA BASE, FORMA DE PAGAMENTO,NMERO DE PARCELAS*/
   PROCEDURE SET_PARAMETROS (P_CDCLIEFORN IN VARCHAR2,
                             P_IDENTRSAIDOP IN VARCHAR2,
                             P_DTEMISSAO IN DATE,
                             P_DTENTRSAID IN DATE,
                             P_CDFILDEST IN VARCHAR2,
                             P_CDFILLANC IN VARCHAR2,
                             P_NRAUTOFORN IN VARCHAR2,
                             P_CDFILPEDIDO IN VARCHAR2,
                             P_NRPEDIDO IN VARCHAR2,
                             P_CDSERVCONT IN VARCHAR2,
                             P_INDIAS OUT NUMBER,
                             P_DTBASE OUT DATE,
                             P_IDFORMAPGTO OUT VARCHAR2,
                             P_IDTIPODTPGT OUT VARCHAR2,
                             P_IDTPPERIODO OUT VARCHAR2,
                             P_NRPARCELAS OUT VARCHAR2,
                             P_NRPARCPGTO OUT VARCHAR2,
                             P_CDFORMPGTO OUT VARCHAR2,
                             P_CODIGO IN VARCHAR2)
   IS
      V_IDTPFORPGT  VARCHAR2(1);
      V_IDUTDIAPGT  VARCHAR2(1);
      V_NRDIAFIXO   VARCHAR2(2);
      V_DTAUX DATE;
      V_BOCONSFERIBANC BOOLEAN;
      /*PARMETROS GERAIS PARA GERAO DA DATA BASE E QUANTIDADE DE DIAS*/
      CURSOR C_PARAMETRIZACAO_PGTO (P_CLIEFORN VARCHAR2,
                                    P_CODIGO VARCHAR2,
                                    P_CDSERVCONT VARCHAR2,
                                    P_CDFILIDEST VARCHAR2,
                                    P_CDFILILANC VARCHAR2,
                                    P_NRAUTOFORN VARCHAR2,
                                    P_CDFILPEDIDO VARCHAR2,
                                    P_NRPEDIDO VARCHAR2)
          IS
         SELECT IDTPFORPGT, IDUTDIAPGT, NRDIAPGFIX, IDTIPODTPGT, CDFORMPGT, IDTPPERIODO, NRPARCPGTO
           FROM
          (SELECT F.IDTPFORPGTFO IDTPFORPGT, F.IDUTDIAPGTFO IDUTDIAPGT, F.NRDIAPGFIXFO NRDIAPGFIX,
                  FP.IDTIPODTPGTO IDTIPODTPGT, F.CDFORMPGTO CDFORMPGT, FP.IDTPPERIODO IDTPPERIODO,
                  F.NRPARCPGTOFO NRPARCPGTO
             FROM FORNECEDOR F, FORMPGTO FP
            WHERE (F.CDFORNECED = P_CDCLIEFORN)
              AND (F.CDFORMPGTO = FP.CDFORMPGTO(+))
              AND (P_CODIGO = '0')
            UNION
           SELECT C.IDTPFORPGTCL IDTPFORPGT, C.IDUTDIAPGTCL IDUTDIAPGT , C.NRDIAPGFIXCL NRDIAPGFIX ,
                  FP.IDTIPODTPGTO IDTIPODTPGT, C.CDFORMPGTO CDFORMPGT, FP.IDTPPERIODO IDTPPERIODO,
                  C.NRPARCPGTOCL NRPARCPGTO
             FROM CLIENTE C, FORMPGTO FP
            WHERE (C.CDCLIENTE = P_CDCLIEFORN)
              AND (C.CDFORMPGTO = FP.CDFORMPGTO(+))
              AND (P_CODIGO = '1')
            UNION
           SELECT SC.IDTPFORPGTSC IDTPFORPGT, SC.IDUTDIAPGTSC IDUTDIAPGT, SC.NRDIAPGFIXSC NRDIAPGFIX,
                  FP.IDTIPODTPGTO IDTIPODTPGT, SC.CDFORMPGTO CDFORMPGT, FP.IDTPPERIODO IDTPPERIODO,
                  SC.NRPARCPGTOSC NRPARCPGTO
             FROM SERVCONTRAT SC, FORMPGTO FP
            WHERE (SC.CDCLIENTE  = P_CDCLIEFORN)
              AND (SC.CDSERVCONT = P_CDSERVCONT)
              AND (SC.CDFORMPGTO = FP.CDFORMPGTO(+))
              AND (P_CODIGO = '2')
            UNION
           SELECT A.IDTPFORPGTAF IDTPFORPGT, A.IDUTDIAPGTAF IDUTDIAPGT, A.NRDIAPGFIXAF NRDIAPGFIX,
                  FP.IDTIPODTPGTO IDTIPODTPGT, A.CDFORMPGTO CDFORMPGT, FP.IDTPPERIODO IDTPPERIODO,
                  NRPARCPGTOAF NRPARCPGTO
             FROM AUTOFORNE A, FORMPGTO FP
            WHERE (A.CDFILDEST  = P_CDFILDEST)
              AND (A.CDFILLANC  = P_CDFILLANC)
              AND (A.NRAUTOFORN = P_NRAUTOFORN)
              AND (A.CDFORMPGTO = FP.CDFORMPGTO(+))
              AND (P_CODIGO = '3')
            UNION
           SELECT P.IDTPFORPGTPED IDTPFORPGT, P.IDUTDIAPAGPED IDUTDIAPGT, P.NRDIAPGFIXPED NRDIAPGFIX,
                  FP.IDTIPODTPGTO IDTIPODTPGT, P.CDFORMPGTO CDFORMPGT, FP.IDTPPERIODO IDTPPERIODO,
                  NRPARCPGTOPED NRPARCPGTO
             FROM PEDIDO P, FORMPGTO FP
            WHERE (P.CDFILIAL   = P_CDFILPEDIDO)
              AND (P.NRPEDIDO   = P_NRPEDIDO)
              AND (P.CDFORMPGTO = FP.CDFORMPGTO(+))
              AND (P_CODIGO = '4'));

      CURSOR C_QTPARCELAS(P_CLIEFORN VARCHAR2,
                          P_CODIGO VARCHAR2,
                          P_CDSERVCONT VARCHAR2,
                          P_CDFILIDEST VARCHAR2,
                          P_CDFILILANC VARCHAR2,
                          P_NRAUTOFORN VARCHAR2,
                          P_CDFILPEDIDO VARCHAR2,
                          P_NRPEDIDO VARCHAR2)
          IS
          SELECT MAX(QTPARCELAS)  AS QTPARCELAS
            FROM
            (SELECT COUNT(*) AS QTPARCELAS
               FROM PRAZOPGTFORN
              WHERE CDFORNECED = P_CDCLIEFORN
                AND P_CODIGO = '0'
              UNION
             SELECT COUNT(*) AS QTPARCELAS
               FROM PRAZOPGTCLIE
              WHERE CDCLIENTE = P_CDCLIEFORN
                AND P_CODIGO = '1'
              UNION
             SELECT COUNT(*) AS QTPARCELAS
               FROM PZPGSERCONT
              WHERE CDCLIENTE = P_CDCLIEFORN
                AND CDSERVCONT = P_CDSERVCONT
                AND P_CODIGO = '2'
              UNION
             SELECT COUNT(*) AS QTPARCELAS
               FROM PRAZOPGTAF
              WHERE CDFILDEST  = P_CDFILDEST
                AND CDFILLANC  = P_CDFILLANC
                AND NRAUTOFORN = P_NRAUTOFORN
                AND P_CODIGO = '3'
              UNION
             SELECT COUNT(*) AS QTPARCELAS
               FROM PRAZOPGTOPED
              WHERE CDFILIAL = P_CDFILPEDIDO
                AND NRPEDIDO = P_NRPEDIDO
                AND P_CODIGO = '4');
   BEGIN
      OPEN C_PARAMETRIZACAO_PGTO(P_CDCLIEFORN,
                                 P_CODIGO,
                                 P_CDSERVCONT,
                                 P_CDFILDEST,
                                 P_CDFILLANC,
                                 P_NRAUTOFORN,
                                 P_CDFILPEDIDO,
                                 P_NRPEDIDO);
      FETCH C_PARAMETRIZACAO_PGTO INTO
         V_IDTPFORPGT,
         V_IDUTDIAPGT,
         V_NRDIAFIXO,
         P_IDTIPODTPGT,
         P_CDFORMPGTO,
         P_IDTPPERIODO,
         P_NRPARCPGTO;

      P_IDFORMAPGTO := V_IDTPFORPGT;



      IF NOT (V_IDTPFORPGT IS NULL) AND C_PARAMETRIZACAO_PGTO%FOUND THEN
         IF V_IDTPFORPGT = 'V' THEN
            /*QUANTIDADE DE PARCELAS*/
            OPEN C_QTPARCELAS(P_CDCLIEFORN,
                              P_CODIGO,
                              P_CDSERVCONT,
                              P_CDFILDEST,
                              P_CDFILLANC,
                              P_NRAUTOFORN,
                              P_CDFILPEDIDO,
                              P_NRPEDIDO);
            FETCH C_QTPARCELAS INTO P_NRPARCELAS;
         ELSE
            P_NRPARCELAS := P_NRPARCPGTO;
         END IF;
         /*DETEMINA A DATA BASE E O NMERO DE DIAS A PARTIR DO TIPO DA FORMA DE PAGAMENTO
           E DO TIPO DA DATA DE PAGAMENTO*/
         FORMAPGTO(P_INDIAS,
                   P_DTBASE,
                   P_DTEMISSAO,
                   P_DTENTRSAID,
                   V_IDTPFORPGT,
                   P_IDTIPODTPGT,
                   V_IDUTDIAPGT,
                   V_NRDIAFIXO,
                   P_CODIGO,
                   P_IDTIPODTPGT,
                   P_CDFORMPGTO);
      ELSE
         P_INDIAS := 0;
         P_DTBASE := P_DTEMISSAO;
         P_IDFORMAPGTO := NULL;
         P_IDTIPODTPGT := NULL;
         P_IDTPPERIODO := NULL;
         P_NRPARCELAS := 1;
         P_NRPARCPGTO := 1;
      END IF;

      CLOSE C_PARAMETRIZACAO_PGTO;
      IF C_QTPARCELAS%ISOPEN THEN
         CLOSE C_QTPARCELAS;
      END IF;
   END;
BEGIN
   V_CDCLIEFORN := P_CDCLIEFORN;
   /* DETERMINA O CDIGO PARA PESQUISA DOS DADOS PARA VENCIMENTO :
   0 - FORNECEDOR
   1 - CLIENTE
   2 - CONTRATO DE SERVIO
   3 - AUTORIZAO DE FORNECIMENTO
   4 - PEDIDO   */

   IF P_IDENTRSAIDOP = 'E' THEN
      V_CODIGO := '0';
   ELSIF P_IDENTRSAIDOP = 'S' THEN
      /*BUSCA O CONTRATO DE SERVIO RELACIONADO  NOTA*/
      BEGIN
         SELECT MAX(CDSERVCONT) AS CDSERVCONT
           INTO V_CDSERVCONT
           FROM SERVFATU
          WHERE (CDFILIAL   = P_CDFILIAL) AND
                (NRLANCTONF = P_NRLANCTONF);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
               V_CDSERVCONT := NULL;
      END;
      IF (V_CDSERVCONT IS NULL) THEN
         V_CODIGO := '1';
      ELSE
         V_CODIGO := '2';
      END IF;
      /*VERIFICA SE  DEVOLUO AO FORNECEDOR*/
      IF P_ORIGDESTNF = 'F' THEN
         /* BUSCA O CLIENTE RELACIONADO AO FORNECEDOR NO CADASTRO DE FORNECEDOR
          APENAS PARA NOTA DE DEVOLUO */
         BEGIN
            SELECT CDCLIENTE
              INTO V_CDCLIEFORN
              FROM FORNECEDOR
             WHERE CDFORNECED = P_CDCLIEFORN;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CDCLIEFORN := NULL;
         END;
         V_CODIGO := '2';
      END IF;
   ELSIF P_IDENTRSAIDOP = 'A' THEN
      V_CODIGO := '3';
   ELSIF P_IDENTRSAIDOP = 'P' THEN
      V_CODIGO := '4';
   END IF;

   /* VERIFICA GERAO DE PARCELA DE SUBST. TRIBUTRIA*/
   IF (V_CODIGO = '1') OR (V_CODIGO = '4') THEN
      BEGIN
         SELECT NVL(IDGERPARCST, 'N')
           INTO V_IDGERPARCST
           FROM PARAMFILIAL
          WHERE CDFILIAL = P_CDFILIEMIRECE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_IDGERPARCST := 'N';
      END;
   ELSE
      V_IDGERPARCST := 'N';
   END IF;

   /*VALOR DA PARCELA DE SUBST. TRIBUTRIA*/
   V_VRTOTIMPONF := 0;
   IF V_IDGERPARCST = 'S' THEN
      BEGIN
         SELECT NVL(N.VRTOTIMPONF, 0)
           INTO V_VRTOTIMPONF
           FROM IMPOSNFTOT N, IMPOSTO I, IMPOSTO I2
          WHERE N.CDFILIAL = P_CDFILIAL
            AND N.NRLANCTONF = P_NRLANCTONF
            AND N.CDIMPOSTO = I.CDIMPOSTRIB
            AND I.CDIMPOSTRIB = I2.CDIMPOSTO
            AND I2.IDTRATIMPO = 'A';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRTOTIMPONF := 0;
      END;
   END IF;

   V_INDIAS := 0;
   V_DTVENCIMENTO := NULL;

   /*DETERMINA A DATA BASE E O NMERO DE DIAS APS A EMISSO*/
   SET_PARAMETROS(V_CDCLIEFORN,
                  P_IDENTRSAIDOP,
                  P_DTEMISSAO,
                  P_DTENTRSAID,
                  P_CDFILDEST,
                  P_CDFILLANC,
                  P_NRAUTOFORN,
                  P_CDFILPEDIDO,
                  P_NRPEDIDO,
                  V_CDSERVCONT,
                  V_INGUARDANRDIAS,
                  V_DTBASE,
                  V_IDFORMAPGTO,
                  V_IDTIPODTPGT,
                  V_IDTPPERIODO,
                  V_NRPARCELAS,
                  V_NRPARCPGTO,
                  V_CDFORMPGTO,
                  V_CODIGO);

   V_BOCONSFERIBANC := (V_IDTIPODTPGT = 'UEMI') OR (V_IDTIPODTPGT = 'UEME');
   /*ABATE A PARCELA DE IMPOSTO*/
   V_VRPAGTO := ROUND(P_VRPAGTO - V_VRTOTIMPONF, 2);

   IF V_IDFORMAPGTO IS NULL THEN /* SEM FORMA DE PAGAMENTO*/
      V_NRPARCELAS := 1;

      INSERT INTO TRAB_VENCIMENTO
         (CDFILIALVENC, NRLANCTONFVENC, DTVENCIMENTO, VRVENCIMENTO, QTDIASVENC, NRSEQVENC)
      VALUES
         (P_CDFILIAL, P_NRLANCTONF, P_DTEMISSAO, V_VRPAGTO, 0, LPAD(TO_CHAR(1),5, '0'));

   ELSIF (V_IDFORMAPGTO = 'F') THEN /*FORMA DE PAGAMENTO FIXA*/
      V_DTVENCIMENTO := V_DTBASE + V_INGUARDANRDIAS;
      V_VRPARCVENC := ROUND(V_VRPAGTO/V_NRPARCPGTO,2);
      V_SEQUENCIAL := 1;
      V_NRPARCELAS := V_NRPARCPGTO;
      /*LANA CONTAS PARA O MESMO DIA INDICADO PORM EM MESES DIFERENTES*/
      WHILE V_SEQUENCIAL <= V_NRPARCELAS LOOP
         IF V_SEQUENCIAL = V_NRPARCELAS THEN
            V_VRPARCVENC := V_VRPARCVENC + ROUND(V_VRPAGTO - (V_VRPARCVENC * V_NRPARCPGTO),2);
         END IF;
         /*INSERE OS DADOS DO VENCIMENTO*/
         INSERT INTO TRAB_VENCIMENTO
            (CDFILIALVENC, NRLANCTONFVENC, DTVENCIMENTO, VRVENCIMENTO, QTDIASVENC, NRSEQVENC)
         VALUES
            (P_CDFILIAL, P_NRLANCTONF, V_DTVENCIMENTO, V_VRPARCVENC, NVL(TRUNC(V_DTVENCIMENTO-P_DTEMISSAO), 0), LPAD(TO_CHAR(V_SEQUENCIAL), 5, '0'));

         IF NOT (V_DTVENCIMENTO IS NULL) THEN
            V_DTVENCIMENTO := ADD_MONTHS(V_DTVENCIMENTO, 1);
         END IF;

         V_SEQUENCIAL := V_SEQUENCIAL + 1;
      END LOOP;
   ELSE /*FORMA DE PAGAMENTO VARIVEL*/
      V_SEQUENCIAL := 1;
      V_VRPARCAUXI := 0;

      /*PARCELAS PARA GERAO DE TIPO DE PAGAMENTO VARIVEL*/
      FOR C_PARC IN C_PARCELAS(V_CDCLIEFORN,
                               V_CODIGO,
                               V_CDSERVCONT,
                               P_CDFILDEST,
                               P_CDFILLANC,
                               P_NRAUTOFORN,
                               P_CDFILPEDIDO,
                               P_NRPEDIDO) LOOP

         V_VRPARCVENC := ROUND((V_VRPAGTO * (C_PARC.VRPERCPGTO/100)),2);
         V_VRPARCAUXI := V_VRPARCAUXI + V_VRPARCVENC;

         IF V_SEQUENCIAL = V_NRPARCELAS THEN
            V_VRPARCVENC := V_VRPARCVENC + ROUND((V_VRPAGTO - V_VRPARCAUXI),2);
         END IF;

         IF C_PARC.NRDIASPGTO = 0 THEN
            V_INDIAS := V_INGUARDANRDIAS +1;
         ELSE
            V_INDIAS := V_INGUARDANRDIAS + C_PARC.NRDIASPGTO;
         END IF;

         IF NOT V_DTBASE IS NULL THEN
            V_DTVENCIMENTO := V_DTBASE + V_INDIAS;
         END IF;

         IF V_BOCONSFERIBANC THEN
            CONSISTE_FERI_BANC(P_CDFILDEST,
                               V_DTVENCIMENTO,
                               V_DTVENCIMENTO,
                               V_IDTIPODTPGT,
                               P_DTEMISSAO);
         END IF;
         /*CONSIDERA DIA DE PAGAMENTO. CASO FOR NOTA FISCAL DE SERVIO, O DIA DE PAGAMENTO J FOI CONSISTIDO ANTERIORMENTE*/
         IF NOT (V_DTVENCIMENTO IS NULL) AND NOT (V_CODIGO = '4') THEN
            V_DTVENCIMENTO := DIAPGTO(V_DTVENCIMENTO,
                                      V_IDTPPERIODO,
                                      V_CDFORMPGTO);
         END IF;

         INSERT INTO TRAB_VENCIMENTO
            (CDFILIALVENC, NRLANCTONFVENC, DTVENCIMENTO, VRVENCIMENTO, QTDIASVENC, NRSEQVENC)
         VALUES
            (P_CDFILIAL, P_NRLANCTONF, V_DTVENCIMENTO, V_VRPARCVENC, NVL(TRUNC(V_DTVENCIMENTO-P_DTEMISSAO), 0), LPAD(TO_CHAR(V_SEQUENCIAL), 5, '0'));

         V_SEQUENCIAL := V_SEQUENCIAL + 1;
      END LOOP;

      IF V_SEQUENCIAL = 1 THEN /*FORMA DE PAGAMENTO VARIVEL NO INFORMADA. LANAMENTO  VISTA*/
         INSERT INTO TRAB_VENCIMENTO
            (CDFILIALVENC, NRLANCTONFVENC, DTVENCIMENTO, VRVENCIMENTO, QTDIASVENC, NRSEQVENC)
         VALUES
            (P_CDFILIAL, P_NRLANCTONF, P_DTEMISSAO, V_VRPAGTO, 0, LPAD(TO_CHAR(1),5, '0'));
      END IF;
      
   END IF;

   /*LANA VENCIMENTO PARA O VALOR DA SUBST. TRIBUTRIA*/
   IF NOT (V_VRTOTIMPONF = 0) THEN
      IF V_DTVENCIMENTO IS NULL THEN
         V_DTVENCIMENTO := P_DTEMISSAO;
      END IF;

      INSERT INTO TRAB_VENCIMENTO
        (CDFILIALVENC, NRLANCTONFVENC, DTVENCIMENTO, VRVENCIMENTO, QTDIASVENC, NRSEQVENC)
      VALUES
        (P_CDFILIAL, P_NRLANCTONF, V_DTVENCIMENTO, V_VRTOTIMPONF, NVL(TRUNC(V_DTVENCIMENTO-P_DTEMISSAO), 0), LPAD(TO_CHAR(V_NRPARCELAS + 1), 5, '0'));
   END IF;

   IF C_PARCELAS%ISOPEN THEN
      CLOSE C_PARCELAS;
   END IF;
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('CRIA_VENCIMENTO','5.09.xxx'); END;
/
